==> 3.2.0.tar.gz: test/method_call_handler/verify_user_test.dart <==
// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/verify_user_test_data.dart';
import '../util/ios_test_init.dart';
import '../util/method_channel_native_call_mock.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    const MethodChannel channel = MethodChannel(
      CommonTestData.methodChannelName,
    );

    group("Given verifyUser is called from the native side", () {
      const methodName = 'verifyUser';
      late MockMethodChannelMethodCallCache mockCache;
      group("When the incoming parameters are correct and for pin verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final MockAuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyPinCorrectArguments,
          );

          verifyInOrder([
            mockCache.getOperation(operationId),
            mockCache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            mockOperation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for pin verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final AuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyPinWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(mockCache.getOperation(operationId));
        });
      });

      group(
          "When the incoming parameters are correct and for fingerprint verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final MockAuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyFingerprintCorrectArguments,
          );

          verifyInOrder([
            mockCache.getOperation(operationId),
            mockCache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            mockOperation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for fingerprint verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final AuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyFingerprintWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(mockCache.getOperation(operationId));
        });
      });

      group(
          "When the incoming parameters are correct and for biometric verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final MockAuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyBiometricCorrectArguments,
          );

          verifyInOrder([
            mockCache.getOperation(operationId),
            mockCache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            mockOperation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for biometric verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final AuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyBiometricWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(mockCache.getOperation(operationId));
        });
      });

      group("When the operationId is missing", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final operationId = CommonTestData.wrongId;
          final UserInteractionPlatformOperation mockOperation =
              MockUserInteractionPlatformOperation();

          when(mockCache
                  .getOperation<UserInteractionPlatformOperation>(operationId))
              .thenReturn(mockOperation);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenThrow(PlatformChannelException(
              error: PlatformChannelError.noOperationFoundError(
                  operationId: operationId)));

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.wrongIdArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(
              e.message,
              CommonTestData.platformChannelExceptionMatcher,
            );
          } on Exception {
            fail('PlatformException expected');
          }
          verify(
            mockCache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
          );
        });
      });
    });
  });
}

==> 3.4.0.tar.gz: test/method_call_handler/verify_user_test.dart <==
// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/verify_user_test_data.dart';
import '../util/ios_test_init.dart';
import '../util/method_channel_native_call_mock.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    late MethodChannel channel;
    late MockMethodChannelMethodCallCache cache;
    const methodName = 'verifyUser';

    setUp(() {
      cache = MockMethodChannelMethodCallCache();
      NevisMobileAuthenticationSdkIos.mock(
          MethodChannelMethodCallHandler.mock(cache));
      channel = const MethodChannel(
        CommonTestData.methodChannelName,
      );
    });

    tearDown(() {
      channel.setMockMethodCallHandler(null);
      resetMockitoState();
    });

    group("Given $methodName is called from the native side", () {
      group("When the incoming parameters are correct and for pin verification",
          () {
        test("Then the call is propagated to the cached delegate", () async {
          final operation = MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(cache.getOperation(operationId)).thenReturn(operation);
          when(cache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(cache.removeOperation(operationId)).thenReturn(null);
          when(operation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(operation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyPinCorrectArguments,
          );

          verifyInOrder([
            cache.getOperation(operationId),
            cache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            operation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for pin verification",
          () {
        test("Then PlatformChannelException is thrown", () async {
          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyPinWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(cache.getOperation(any));
        });
      });

      group(
          "When the incoming parameters are correct and for fingerprint verification",
          () {
        test("Then the call is propagated to the cached delegate", () async {
          final operation = MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(cache.getOperation(operationId)).thenReturn(operation);
          when(cache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(cache.removeOperation(operationId)).thenReturn(null);
          when(operation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(operation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyFingerprintCorrectArguments,
          );

          verifyInOrder([
            cache.getOperation(operationId),
            cache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            operation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for fingerprint verification",
          () {
        test("Then PlatformChannelException is thrown", () async {
          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyFingerprintWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(cache.getOperation(any));
        });
      });

      group(
          "When the incoming parameters are correct and for biometric verification",
          () {
        test("Then the call is propagated to the cached delegate", () async {
          final operation = MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(cache.getOperation(operationId)).thenReturn(operation);
          when(cache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(cache.removeOperation(operationId)).thenReturn(null);
          when(operation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(operation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyBiometricCorrectArguments,
          );

          verifyInOrder([
            cache.getOperation(operationId),
            cache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            operation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for biometric verification",
          () {
        test("Then PlatformChannelException is thrown", () async {
          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyBiometricWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(cache.getOperation(any));
        });
      });

      group("When the operationId is missing", () {
        test("Then PlatformChannelException is thrown", () async {
          final operationId = CommonTestData.wrongId;
          final operation = MockUserInteractionPlatformOperation();
          when(cache
                  .getOperation<UserInteractionPlatformOperation>(operationId))
              .thenReturn(operation);
          when(operation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(operation);
          when(cache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenThrow(
            PlatformChannelException(
              error: PlatformChannelError.noOperationFoundError(
                operationId: operationId,
              ),
            ),
          );

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.wrongIdArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(
              e.message,
              CommonTestData.platformChannelExceptionMatcher,
            );
          } on Exception {
            fail('PlatformException expected');
          }
          verify(
            cache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
          );
        });
      });
    });
  });
}
